package com.diagbot.util;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.dto.PushBaseDTO;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.model.entity.BodyPart;
import com.diagbot.model.entity.Clinical;
import com.diagbot.model.entity.Negative;
import com.diagbot.model.entity.PD;
import com.diagbot.model.entity.Usual;
import com.diagbot.model.entity.Vital;
import com.diagbot.model.label.VitalLabel;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @description: core工具类
 * @author: zhoutg
 * @time: 2020/7/30 15:18
 */
@Slf4j
@Component
public class CoreUtil {

    public static Boolean debugFlag;

    @Value("${debugFlag}")
    public void setDebugFlag(Boolean debugFlag) {
        CoreUtil.debugFlag = debugFlag;
    }

    /**
     * 将列表中属性对应的值以list形式返回
     *
     * @param list
     * @param propertyName
     * @param <T>
     * @return
     */
    public static <T> List<String> getPropertyList(List<T> list, String propertyName) {
        List<String> res = new ArrayList<>();
        if (ListUtil.isEmpty(list)) {
            return res;
        }
        for (T t : list) {
            try {
                String val = (String) getFieldValue(t, propertyName);
                if (StringUtil.isNotBlank(val)) {
                    res.add(val);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return res;
    }

    /**
     * 将列表中属性(name)对应的值以list形式返回
     *
     * @param list
     * @param <T>
     * @return
     */
    public static <T> List<String> getPropertyList(List<T> list) {
        return getPropertyList(list, "name");
    }

    /**
     * 将列表中属性对应的值从map中获取，如果有值，将standName替换
     *
     * @param list
     * @param propertyName
     * @param <T>
     * @return
     */
    public static <T> List<String> setPropertyList(List<T> list, String propertyName, String standName, Map<String, String> map) {
        List<String> res = new ArrayList<>();
        if (ListUtil.isEmpty(list)) {
            return res;
        }
        for (T t : list) {
            try {
                setFieldValue(t, propertyName, standName, map);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return res;
    }

    public static <T> void setPropertyList(T list, String propertyName, String standName, Map<String, String> map) {
        if (list == null) {
            return;
        }
        try {
            setFieldValue(list, propertyName, standName, map);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 将列表中属性对应的值从map中获取，如果有值，将standName替换
     *
     * @param list
     * @param <T>
     * @return
     */
    public static <T> List<String> setPropertyList(List<T> list, Map<String, String> map) {
        return setPropertyList(list, "name", "standName", map);
    }

    /**
     * 循环向上转型, 获取对象的 DeclaredField
     *
     * @param object    : 子类对象
     * @param fieldName : 父类中的属性名
     * @return 父类中的属性对象
     */
    public static Field getDeclaredField(Object object, String fieldName) {
        Field field = null;
        Class<?> clazz = object.getClass();
        for (; clazz != Object.class; clazz = clazz.getSuperclass()) {
            try {
                field = clazz.getDeclaredField(fieldName);
                return field;
            } catch (Exception e) {
                //这里甚么都不要做！并且这里的异常必须这样写，不能抛出去。
                //如果这里的异常打印或者往外抛，则就不会执行clazz = clazz.getSuperclass(),最后就不会进入到父类中了
            }
        }
        return null;
    }

    /**
     * 获取对象的属性值,直接使用getDeclaredFields()方法只能获取当前类声明的字段，需要递归向上获取
     *
     * @param object    : 子类对象
     * @param fieldName : 父类中的属性名
     * @return : 父类中的属性值
     */
    public static Object getFieldValue(Object object, String fieldName) {
        try {
            //根据 对象和属性名通过反射 调用上面的方法获取 Field对象
            Field field = getDeclaredField(object, fieldName);
            if (field == null) {
                return null;
            }
            //抑制Java对其的检查
            field.setAccessible(true);
            //获取 object 中 field 所代表的属性值
            return field.get(object);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 直接设置对象属性值, 忽略 private/protected 修饰符, 也不经过 setter
     *
     * @param object    : 子类对象
     * @param name      : 父类中的属性名
     * @param standName : 要替换的属性名
     * @param value     : 将要设置的值
     */
    public static void setFieldValue(Object object, String name, String standName, Map<String, String> value) {
        //根据 对象和属性名通过反射 调用上面的方法获取 Field对象
        Field field = getDeclaredField(object, name);
        if (field == null) {
            return;
        }
        //抑制Java对其的检查field.setAccessible(true);
        field.setAccessible(true);
        try {
            //将 object 中 field 所代表的值 设置为 value
            String key = (String) field.get(object);
            if (value != null && value.get(key) != null) {
                Field standField = getDeclaredField(object, standName);
                standField.setAccessible(true);
                if ("uniqueName".equals(standName)) {
                    String uniqueName = (String) standField.get(object);
                    if (StringUtil.isBlank(uniqueName)) {
                        standField.set(object, value.get(key));
                    }
                } else {
                    standField.set(object, value.get(key));
                }
            }
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }

    }

    public static <T> List<String> setPropertyList(List<T> list, String name, String detailName, String uniqueName, Map<String, String> map) {
        List<String> res = new ArrayList<>();
        if (ListUtil.isEmpty(list)) {
            return res;
        }
        for (T t : list) {
            try {
                setFieldValue(t, name, detailName, uniqueName, map);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return res;
    }

    public static void setFieldValue(Object object, String name, String standName, String unique, Map<String, String> value) {
        //根据 对象和属性名通过反射 调用上面的方法获取 Field对象
        Field field = getDeclaredField(object, name);
        Field field1 = getDeclaredField(object, standName);
        //抑制Java对其的检查
        field.setAccessible(true);
        field1.setAccessible(true);

        try {
            //将 object 中 field 所代表的值 设置为 value
            String key = (String) field.get(object);
            String key1 = (String) field1.get(object);
            //            String lis_c = key+key1;
            String lis_c = key1;
            if (value != null && value.get(lis_c) != null) {
                Field standField = getDeclaredField(object, unique);
                standField.setAccessible(true);
                standField.set(object, value.get(lis_c));
            }
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }

    }

    /**
     * 将列表中uniqueName为空的值，用其他属性值赋值
     *
     * @param list
     * @param targetProperty
     * @param <T>
     */
    public static <T> void setUninameFromDetail(List<T> list, String targetProperty) {
        for (T r : list) {
            String value = (String)getFieldValue(r, "uniqueName");
            if (StringUtil.isBlank(value)) {
                String detailName = (String) getFieldValue(r, targetProperty);
                ReflectUtil.setProperty(r, "uniqueName", detailName);
            }
        }
    }

    /**
     * 将对象添加到列表中
     *
     * @param list
     * @param t
     * @param <T>
     */
    public static <T> void addBeanToList(List<T> list, T t) {
        if (list == null) {
            list = new ArrayList<T>();
        }
        if (t != null) {
            list.add(t);
        }
    }

    /**
     * 比较化验是否匹配
     *
     * @param ruleBaseDTO
     * @param lisList
     * @return
     */
    public static Map<String, Object> compareLis(RuleBaseDTO ruleBaseDTO, List<Lis> lisList) {
        Map<String, Object> map = new LinkedHashMap<>();
        boolean flag = false;
        if (ruleBaseDTO == null || ListUtil.isEmpty(lisList)) {
            map.put("flag", flag);
            return map;
        }
        List<String> msgList = new ArrayList<>();
        for (Lis lis : lisList) {
            Map mapLis = CoreUtil.compareLis(ruleBaseDTO, lis);
            if (CoreUtil.getMapFlag(mapLis)) {
                flag = true;
                msgList.add((String) mapLis.get("msg"));
            }
        }
        map.put("flag", flag);
        map.put("msgList", msgList);
        return map;
    }

    /**
     * 比较化验是否匹配
     *
     * @param ruleBaseDTO
     * @param lis
     * @return
     */
    public static Map<String, Object> compareLis(RuleBaseDTO ruleBaseDTO, Lis lis) {
        Map<String, Object> map = new LinkedHashMap<>();
        boolean flag = false;
        if (ruleBaseDTO == null || lis == null) {
            map.put("flag", flag);
            return map;
        }
        String uniqueName = lis.getUniqueName();
        // 标准名称相同
        if (StringUtil.isNotBlank(uniqueName) && uniqueName.equals(ruleBaseDTO.getBaseLibName())) {
            if (StringUtil.isNotBlank(lis.getOtherValue())) {
                if (lis.getOtherValue().equals(ruleBaseDTO.getBaseEqValue())) {
                    // 套餐和明细名称一样，提示语只取其中一个
                    if (StringUtil.isNotBlank(lis.getName()) && lis.getName().equals(lis.getDetailName())) {
                        // 有医院原值就提示医院值
                        if (StringUtil.isNotBlank(lis.getResult())) {
                            map.put("msg", lis.getName() + lis.getDetailName() + lis.getResult());
                        } else {
                            map.put("msg", lis.getName() + lis.getDetailName() + lis.getOtherValue());
                        }
                    } else {
                        // 有医院原值就提示医院值
                        if (StringUtil.isNotBlank(lis.getResult())) {
                            map.put("msg", lis.getDetailName() + lis.getResult());
                        } else {
                            map.put("msg", lis.getDetailName() + lis.getOtherValue());
                        }
                    }
                    flag = true;
                }
            } else if (lis.getValue() != null) {
                double value = lis.getValue();
                flag = compareNum(ruleBaseDTO, value);
                if (flag) {
                    if (StringUtil.isNotBlank(lis.getName()) && lis.getName().equals(lis.getDetailName())) {
                        if (StringUtil.isNotBlank(lis.getResult())) {
                            map.put("msg", lis.getDetailName() + lis.getResult());
                        } else {
                            map.put("msg", lis.getDetailName() + subZeroAndDot(String.valueOf(lis.getValue())));
                        }
                    } else {
                        if (StringUtil.isNotBlank(lis.getResult())) {
                            map.put("msg", lis.getName() + lis.getDetailName() + lis.getResult());
                        } else {
                            map.put("msg", lis.getName() + lis.getDetailName() + subZeroAndDot(String.valueOf(lis.getValue())));
                        }
                    }
                }
            }
        }
        map.put("flag", flag);
        return map;
    }

    /**
     * 比较查体是否匹配
     *
     * @param ruleBaseDTO
     * @param vitalLabel
     * @return
     */
    public static Map<String, Object> compareVital(RuleBaseDTO ruleBaseDTO, VitalLabel vitalLabel) {
        Map<String, Object> map = new LinkedHashMap<>();
        List<String> msgList = new ArrayList<>();
        map.put("msgList", msgList);
        List<Vital> vitalList = vitalLabel.getVitals(); // 体征数据
        List<Clinical> clinicals = vitalLabel.getClinicals(); // 体征临床表现
        boolean flag = false;
        if (ruleBaseDTO == null) {
            map.put("flag", flag);
            return map;
        }
        // if (StringUtil.isBlank(ruleBaseDTO.getBaseMaxValue()) && StringUtil.isBlank(ruleBaseDTO.getBaseMinValue())) {
        // 如果是体格检查结果，就用临床表现比较；如果是体格检查项目，用体征数值比较
        if (ruleBaseDTO.getBaseLibType().equals(LexiconEnum.VitalResult.getKey())) {
            // 体征名称比较(例如：喉头水肿)
            for (Clinical clinical : clinicals) {
                if (ruleBaseDTO.getBaseLibName().equals(clinical.getStandName())) {
                    msgList.add(clinical.getName());
                    flag = true;
                }
            }
        } else {
            // 体征数值比较(例如：体温，血压)
            for (Vital vital : vitalList) {
                List<Usual> usualList = vital.getUsualList();
                if (ruleBaseDTO.getBaseLibName().equals(vital.getStandName()) && vital.getPd() != null
                        && StringUtil.isNotBlank(vital.getPd().getValue())) {
                    flag = compareNum(ruleBaseDTO, Double.parseDouble(vital.getPd().getValue()));
                    if (flag) {
                        msgList.add(vital.getName() + subZeroAndDot(String.valueOf(vital.getPd().getValue())));
                    }
                } else if (ListUtil.isNotEmpty(usualList)) { // 血压比较
                    for (Usual usual : usualList) {
                        if (ruleBaseDTO.getBaseLibName().equals(usual.getStandName())) {
                            flag = compareNum(ruleBaseDTO, Double.parseDouble(usual.getValue()));
                            if (flag) {
                                msgList.add(vital.getName() + subZeroAndDot(String.valueOf(vital.getPd().getValue())));
                            }
                        }
                    }
                }
            }
        }

        map.put("flag", flag);
        return map;
    }

    /**
     * 比较数值大小（double）
     *
     * @param ruleBaseDTO
     * @param value
     * @return
     */
    public static boolean compareNum(RuleBaseDTO ruleBaseDTO, double value) {
        boolean flag = false;
        Double min = null;
        Double max = null;
        boolean minFlag = false, maxFlag = false;
        if (StringUtil.isNotBlank(ruleBaseDTO.getBaseMinValue())) {
            min = Double.parseDouble(ruleBaseDTO.getBaseMinValue());
        }
        if (StringUtil.isNotBlank(ruleBaseDTO.getBaseMaxValue())) {
            max = Double.parseDouble(ruleBaseDTO.getBaseMaxValue());
        }

        if (max != null) {
            if ("<".equals(ruleBaseDTO.getBaseMaxOperator())) {
                if (value < max) {
                    maxFlag = true;
                }
            } else if ("<=".equals(ruleBaseDTO.getBaseMaxOperator())) {
                if (value <= max) {
                    maxFlag = true;
                }
            }
        }
        if (min != null) {
            if (">".equals(ruleBaseDTO.getBaseMinOperator())) {
                if (value > min) {
                    minFlag = true;
                }
            } else if (">=".equals(ruleBaseDTO.getBaseMinOperator())) {
                if (value >= min) {
                    minFlag = true;
                }
            }
        }
        if (min != null && max != null) {
            if (min > max) { // 取两头数据
                if (minFlag || maxFlag) {
                    flag = true;
                }
            } else { // 取范围数据
                if (minFlag && maxFlag) {
                    flag = true;
                }
            }
        } else if (min != null && max == null) {
            if (minFlag) {
                flag = true;
            }
        } else if (min == null && max != null) {
            if (maxFlag) {
                flag = true;
            }
        }
        return flag;
    }

    /**
     * 去除多余的末尾0
     *
     * @param s
     * @return
     */
    public static String subZeroAndDot(String s) {
        if (s.indexOf(".") > 0) {
            s = s.replaceAll("0+?$", "");//去掉多余的0
            s = s.replaceAll("[.]$", "");//如最后一位是.则去掉
        }
        return s;
    }

    /**
     * list 转 string
     *
     * @param list
     * @return
     */
    public static String listConvertString(List<String> list) {
        return StringUtils.join(list, ",");
    }

    /**
     * 输出调试信息
     *
     * @param start
     * @param msg
     * @param debugMap
     */
    public static void getDebugStr(long start, String msg, Map<String, Object> debugMap) {
        long end = System.currentTimeMillis();
        debugMap.put(msg, +(end - start) / 1000.0 + "秒");
    }

    /**
     * 输出调试信息
     *
     * @param msg
     * @param debugMap
     */
    public static void getDebugObject(String msg, Map<String, Object> debugMap, Object obj) {
        if (debugFlag) {
            debugMap.put(msg, obj);
        }
    }

    /**
     * 返回化验的结果
     *
     * @param lis
     * @return
     */
    public static String getLisValue(Lis lis) {
        if (lis == null) {
            return "";
        }
        return StringUtil.isNotBlank(lis.getOtherValue()) ? lis.getOtherValue() : subZeroAndDot(lis.getValue().toString());
    }

    /**
     * 输出调试信息
     *
     * @param msg
     * @param error
     * @param debugMap
     */
    public static void getDebugStr(String msg, Object error, Map<String, Object> debugMap) {
        debugMap.put(msg, error);
    }

    /**
     * 年龄字符串转换成double类型
     *
     * @param ageStr
     * @return
     */
    public static Double convertAge(String ageStr) {
        try {
            // 防止程序出错
            if (StringUtil.isEmpty(ageStr)) {
                return 20.0;
            }
            // 数值,当成年龄处理
            if (isNumbers(ageStr)) {
                return Double.parseDouble(ageStr);
            }
            // 20日
            if (ageStr.endsWith("日")) {
                return getHalfUp((Double.parseDouble(ageStr.substring(0, ageStr.length() - 1))) / 365);
            }
            // 3岁
            if (ageStr.endsWith("岁")) {
                return Double.parseDouble(ageStr.substring(0, ageStr.length() - 1));
            }
            // 3岁7个月
            if (ageSuiYue(ageStr)) {
                String[] ageArr = new String[2];
                int indexSui = ageStr.indexOf("岁");
                ageArr[0] = ageStr.substring(0, indexSui);
                ageArr[1] = ageStr.substring(indexSui + 1, ageStr.indexOf("个月"));
                return Double.parseDouble(ageArr[0]) + getHalfUp(Double.parseDouble(ageArr[1]) / 12);
            }
            // 1.08月 | .11月 | 3月
            if (ageYue(ageStr)) {
                String noUnit = ageStr.substring(0, ageStr.length() - 1);
                String[] ageArr = new String[2];
                String[] splitArr = noUnit.split("\\.");
                if (splitArr.length == 1) {
                    ageArr[0] = splitArr[0];
                } else if (splitArr.length == 2) {
                    ageArr[0] = splitArr[0];
                    ageArr[1] = splitArr[1];
                }
                Double daySum = 0.0D;
                if (StringUtil.isNotBlank(ageArr[0])) {
                    daySum += Double.parseDouble(ageArr[0]) * 30;
                }
                if (StringUtil.isNotBlank(ageArr[1])) {
                    daySum += Double.parseDouble(ageArr[1]);
                }
                return getHalfUp(daySum / 365);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return 20.0;
        }
        return 20.0;
    }

    /**
     * 判断字符串是否 仅 包含数字
     *
     * @param str
     * @return
     */
    public static boolean numbersOnly(String str) {
        String regex = "^[0-9]+$";
        return str.matches(regex);
    }

    /**
     * 判断字符串是否数值(正)
     *
     * @param str
     * @return
     */
    public static boolean isNumbers(String str) {
        String regex = "[0-9]+|([0-9]+\\.)+[0-9]*|[0-9]*(\\.[0-9]+)+";
        return str.matches(regex);
    }


    /**
     * 判断年龄字符串：xx月
     *
     * @param str
     * @return
     */
    public static boolean ageYue(String str) {
        String regex = "^[0-9]*[\\.]*[0-9]{1,2}月$";
        return str.matches(regex);
    }

    /**
     * 判断年龄字符串：3岁7个月
     *
     * @param str
     * @return
     */
    public static boolean ageSuiYue(String str) {
        String regex = "^[0-9]{1,3}岁[0-9]{1,2}个月$";
        return str.matches(regex);
    }

    /**
     * 四舍五入保留2位小数
     *
     * @param ageStr
     * @return
     */
    public static Double getHalfUp(Double ageStr) {
        BigDecimal bg = new BigDecimal(String.valueOf(ageStr));
        return bg.setScale(3, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    /**
     * 替换首位的标点符号(例如：既往史提取“铁)
     *
     * @param str
     * @return
     */
    public static String replaceStartPunctuation(String str) {
        if (str.startsWith("“")) {
            return str.replaceFirst("“", "");
        }
        return str;
    }

    /**
     * 比较名称是否匹配
     *
     * @param ruleBaseDTO
     * @param input
     * @return
     */
    public static Boolean compareName(RuleBaseDTO ruleBaseDTO, String input) {
        if (StringUtil.isBlank(input) || ruleBaseDTO == null || StringUtil.isBlank(ruleBaseDTO.getBaseLibName())) {
            return false;
        }
        if (input.equals(ruleBaseDTO.getBaseLibName())) {
            return true;
        }
        return false;
    }

    /**
     * 获取所有阳性【T】,转成Item结构，放入列表
     *
     * @param source
     * @param convertList
     */
    public static <T> void addAllConvert(List<Item> source, List<T> convertList) {
        List<Item> other = convertItem(convertList);
        if (ListUtil.isEmpty(other)) {
            return;
        }
        if (source == null) {
            source = new ArrayList<>();
        }
        source.addAll(other);
    }

    /**
     * 将T(包含“negative”，“name”，“standName”字段)转成Item类型
     *
     * @param convertList
     * @return
     */
    public static <T> List<Item> convertItem(List<T> convertList) {
        List<Item> itemList = new ArrayList<>();
        if (ListUtil.isNotEmpty(convertList)) {
            for (T t : convertList) {
                Negative negative = (Negative) CoreUtil.getFieldValue(t, "negative");
                if (negative == null) {
                    Item item = new Item();
                    item.setName((String) CoreUtil.getFieldValue(t, "name"));
                    item.setUniqueName((String) CoreUtil.getFieldValue(t, "standName"));
                    itemList.add(item);
                }
            }
        }
        return itemList;
    }

    /**
     * 获取mapflag值
     *
     * @param map
     * @return
     */
    public static Boolean getMapFlag(Map map) {
        if (map != null && map.get("flag") != null && (Boolean) map.get("flag") == true) {
            return true;
        }
        return false;
    }

    /**
     * 获取mapMsgList值
     *
     * @param map
     * @return
     */
    public static List<String> getMapMsgList(Map map) {
        if (map != null && map.get("flag") != null && (Boolean) map.get("flag") == true) {
            List<String> msgList = (List<String>) map.get("msgList");
            if (ListUtil.isEmpty(msgList)) {
                return new ArrayList<>();
            } else {
                return msgList;
            }
        }
        return new ArrayList<>();
    }

    /**
     * 添加列表
     *
     * @param source
     * @param addList
     * @param <T>
     */
    public static <T> void addList(List<T> source, List<? extends T> addList) {
        if (source == null) {
            return;
        }
        if (ListUtil.isNotEmpty(addList)) {
            source.addAll(addList);
        }
    }

    /**
     * 添加列表
     *
     * @param source
     * @param addList
     * @param <T>
     */
    public static <T> void addSet(Set<T> source, List<? extends T> addList) {
        if (source == null) {
            return;
        }
        if (ListUtil.isNotEmpty(addList)) {
            source.addAll(addList);
        }
    }

    /**
     * 筛选对象列表中“uniqueName”字段为空的数据，返回targetProperty字段列表
     *
     * @param list
     * @param <T>
     * @return
     */
    public static <T> List<String> filterUniqueList(List<T> list, String targetProperty) {
        if (ListUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream()
                .filter(r -> StringUtil.isBlank((String) getFieldValue(r, "uniqueName")))
                .map(r -> (String) getFieldValue(r, targetProperty))
                .collect(Collectors.toList());
    }

    /**
     * 筛选对象列表中“name”字段不为空的数据
     *
     * @param list
     * @param <T>
     * @return
     */
    public static <T> List<String> getName(List<T> list) {
        if (ListUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream()
                .filter(r -> StringUtil.isNotBlank((String) getFieldValue(r, "name")))
                .map(r -> (String) getFieldValue(r, "name"))
                .collect(Collectors.toList());
    }

    /**
     * 筛选对象列表中“name”字段不为空的数据，再拼接：bodyPart + name
     *
     * @param list
     * @param <T>
     * @return
     */
    public static <T> List<String> getNameAppendBodySymptom(List<T> list) {
        List<String> res = new ArrayList<>();
        if (ListUtil.isEmpty(list)) {
            return res;
        }
        for (T t : list) {
            // 症状需要提取名称
            String name = (String) getFieldValue(t, "name");
            if (StringUtil.isNotBlank(name)) {
                res.add(name);
            }
            BodyPart bodyPart = (BodyPart) getFieldValue(t, "bodyPart");
            if (bodyPart != null && StringUtil.isNotBlank(bodyPart.getName())) {
                res.add(bodyPart.getName() + name);
            }
        }
        return res;
    }

    /**
     * 筛选对象列表中“name”字段不为空的数据，再拼接：bodyPart + name
     *
     * @param list
     * @param <T>
     * @return
     */
    public static <T> List<String> getNameAppendBodyVital(List<T> list) {
        List<String> res = new ArrayList<>();
        if (ListUtil.isEmpty(list)) {
            return res;
        }
        for (T t : list) {
            // 刨去体温，脉搏，数值类型的体征数据
            PD pd = (PD) getFieldValue(t, "pd");
            if (pd == null || (pd != null && StringUtil.isNotBlank(pd.getUnit()))) {
                continue;
            }
            String name = (String) getFieldValue(t, "name");
            // 查体拼接结果
            if (StringUtil.isNotBlank(pd.getName())) {
                // 查体结果
                res.add(name + pd.getName());
                // 查体部位 + 查体结果
                BodyPart bodyPart = (BodyPart) getFieldValue(t, "bodyPart");
                if (bodyPart != null && StringUtil.isNotBlank(bodyPart.getName())) {
                    res.add(bodyPart.getName() + name + pd.getName());
                }
            }
        }
        return res;
    }

    /**
     * 筛选对象列表中“uniqueName”字段为空的数据，返回“name”字段列表
     *
     * @param list
     * @param <T>
     * @return
     */
    public static <T> List<String> filterUniqueList(List<T> list) {
        if (ListUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream()
                .filter(r -> StringUtil.isBlank((String) getFieldValue(r, "uniqueName")))
                .map(r -> (String) getFieldValue(r, "name"))
                .collect(Collectors.toList());
    }

    public static <V> Map<String, List<V>> makeEntityListMap(List<V> list, String field1, String field2) {
        Map<String, List<V>> map = new LinkedHashMap();
        if (ListUtil.isNotEmpty(list)) {
            for (V v : list) {
                String value1 = (String) CoreUtil.getFieldValue(v, field1);
                String value2 = (String) CoreUtil.getFieldValue(v, field2);
                String unionKey = "";
                if (StringUtil.isNotBlank(value1)) {
                    unionKey += value1;
                }
                unionKey += "*****";
                if (StringUtil.isNotBlank(value2)) {
                    unionKey += value2;
                }
                List<V> groupList = (List<V>) map.get(unionKey);
                if (groupList == null) {
                    groupList = new ArrayList();
                }
                groupList.add(v);
                map.put(unionKey, groupList);
            }
        }
        return map;
    }

    /**
     * 将字符串以“，”分隔，添加到list列表中
     * @param list
     * @param s
     */
    public static void addSplitString(List<Long> list, String s) {
        if (StringUtil.isBlank(s)) {
            return ;
        }
        list.addAll(Arrays.stream(s.split(",")).map(r -> Long.parseLong(r)).collect(Collectors.toList()));
    }

    /**
     * 在源列表中移除过滤的内容
     *
     * @param orginList
     * @param splitList
     */
    public static void removeRepeat(List<String> orginList, List<String> splitList) {
        if (ListUtil.isEmpty(orginList) || ListUtil.isEmpty(splitList)) {
            return ;
        }
        Iterator<String> iterator = orginList.iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            if (splitList.contains(key)) {
                iterator.remove();
            }
        }
    }

    /**
     * 根据属性名获取内容
     *
     * @param tList
     * @param propertyName
     * @return
     */
    public static <T> List<String> getByPropertyName(List<T> tList, String... propertyName) {
        List<String> list = Lists.newArrayList();
        if (ListUtil.isEmpty(tList)) {
            return list;
        }
        for (T t : tList) {
            for (String key : propertyName) {
                String name = (String)getFieldValue(t, key);
                // 去重
                if (StringUtil.isNotBlank(name) && !list.contains(name)) {
                    list.add(name);
                }
            }
        }
        return list;
    }

    /**
     * 去除重复元素
     *
     * @param list
     */
    public static <T> List<T> removeRepeat(List<T> list) {
        if (ListUtil.isNotEmpty(list) && list.size() > 1) {
            list = list.stream().distinct().collect(Collectors.toList());
        }
        return list;
    }

    /**
     * 根据属性去除重复元素
     *
     * @param source
     * @param property
     * @param <T>
     */
    public static <T> void removeRepeat(List<T> source, String property) {
        if (ListUtil.isEmpty(source) || source.size() < 2) {
            return ;
        }
        List<Object> list = Lists.newArrayList();
        Iterator<T> iterator = source.iterator();
        T t = null;
        Object object = null;
        while(iterator.hasNext()) {
            t = iterator.next();
            object = getFieldValue(t, property);
            if (object != null) {
                if (list.contains(object)) {
                    iterator.remove();
                } else {
                    list.add(object);
                }
            }
        }
    }

    /**
     * 获取指定属性且过滤属性的名称列表，例如：获取对象negative不为null的uniqueName字段值
     *
     * @param input
     * @param propertyName
     * @param filterProperty
     * @param <T>
     * @return
     */
    public static <T> List<String> getPropertyWithFilter(List<T> input, String propertyName, String filterProperty) {
        List<String> res = Lists.newArrayList();
        if (ListUtil.isNotEmpty(input)) {
            for (T d : input) {
                Negative val = (Negative) CoreUtil.getFieldValue(d, filterProperty);
                if (val != null) {
                    String name = (String) CoreUtil.getFieldValue(d, propertyName);
                    if (StringUtil.isNotBlank(name)) {
                        res.add(name);
                    }
                }
            }
        }
        return res;
    }

    public static void main(String[] args) {
        List<PushBaseDTO> pushBaseDTOList = ListUtil.newArrayList();
        PushBaseDTO pushBaseDTO = new PushBaseDTO();
        pushBaseDTO.setName("a1");
        pushBaseDTOList.add(pushBaseDTO);

        PushBaseDTO pushBaseDTO1 = new PushBaseDTO();
        pushBaseDTO1.setName("a1");
        pushBaseDTOList.add(pushBaseDTO1);

        removeRepeat(pushBaseDTOList, "name");
        System.out.println(pushBaseDTOList);

        // List<String> list = Lists.newArrayList("a","b","a");
        // list = removeRepeat(list);
        // System.out.println(list);
        // System.out.println(pushBaseDTOList);


        // List<Item> list = new ArrayList<>();
        // Item item = new Item();
        // item.setUniqueName("test1");
        // item.setName("test");
        // list.add(item);
        //
        // Item item2 = new Item();
        // item2.setUniqueName("test2");
        // item2.setName("test");
        // list.add(item2);
        //
        // Item item3 = new Item();
        // item3.setUniqueName("test1");
        // item3.setName("test");
        // list.add(item3);
        //
        // Item item4 = new Item();
        // item4.setUniqueName("test2");
        // item4.setName("test");
        // list.add(item4);
        // Map<String, List<Item>> map = makeEntityListMap(list, "name", "uniqueName");
        //
        // System.out.println(map);
    }
}
